// PROG6_1.CPP - an example of using timers and counters

// INCLUDES ///////////////////////////////////////////////
#define WIN32_LEAN_AND_MEAN  

#include <windows.h>  
#include <windowsx.h>  
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINCLASS1"
#define WINDOW_WIDTH  320
#define WINDOW_HEIGHT 200

// general defines
#define TIMER_ID1_SEC  1 // id of 1 sec timer
#define TIMER_ID3_SEC  2 // id of 3 sec timer

// GLOBALS ////////////////////////////////////////////////
HWND main_window_handle = NULL; // save the window handle
char buffer[80];                // used to print messages
// FUNCTIONS //////////////////////////////////////////////
LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT		ps;		// used in WM_PAINT
HDC				hdc;	// handle to a device context
static int      counter1=0, // counters
                counter2=0;

// what is the message 
switch(msg)
	{	

    case WM_CREATE:
         {
         // create a 1 second timer
         SetTimer(hwnd, TIMER_ID1_SEC, 1000,NULL);

         // now the 3 second timer
         SetTimer(hwnd, TIMER_ID3_SEC, 3000,NULL);	

         // exit event handler
         return(0);
         } break;

    case WM_TIMER:
         {
         // there is a timer event, test what timer fired
        switch(wparam)
              {
              case TIMER_ID1_SEC:
                   {
                   // print out a message
                   // get the dc
                   hdc = GetDC(hwnd);

                   // set the color
                   SetTextColor(hdc,RGB(0,255,0));
                   SetBkColor(hdc,RGB(0,0,0));
                   SetBkMode(hdc,OPAQUE);

                   // build up the messages
                   sprintf(buffer,"1 second timer has fired %d times",++counter1);

                   // print the message
                   TextOut(hdc,0,0,buffer,strlen(buffer));

                   // release the dc
                   ReleaseDC(hwnd,hdc);
                   } break;

              case TIMER_ID3_SEC:
                   {
                   // make a beep
                   MessageBeep(MB_ICONEXCLAMATION);

                   // get the dc
                   hdc = GetDC(hwnd);

                   // set the color
                   SetTextColor(hdc,RGB(0,255,0));
                   SetBkColor(hdc,RGB(0,0,0));
                   SetBkMode(hdc,OPAQUE);

                   // build up the messages
                   sprintf(buffer,"3 second timer has fired %d times",++counter2);

                   // print the message
                   TextOut(hdc,0,20,buffer,strlen(buffer));

                   // release the dc
                   ReleaseDC(hwnd,hdc);

                   } break;

               // .. test for other id's

               default:break;

               } // end switch
          } break;


	case WM_PAINT: 
		{
		// simply validate the window
		hdc = BeginPaint(hwnd,&ps);	 
   		EndPaint(hwnd,&ps);
		return(0);
   		} break;

	case WM_DESTROY: 
		{
		// kill the application			

        // kill the timers
        KillTimer(hwnd,TIMER_ID1_SEC);
        KillTimer(hwnd,TIMER_ID3_SEC);

		PostQuitMessage(0);
		return(0);
		} break;

	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc

// WINMAIN ////////////////////////////////////////////////
int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{

WNDCLASS winclass;	// this will hold the class we create
HWND	 hwnd;		// generic window handle
MSG		 msg;		// generic message
HDC      hdc;       // generic dc
PAINTSTRUCT ps;     // generic paintstruct
RECT     rect;      // generic rectangle
HBRUSH   hbrush;    // generic brush handle

// first fill in the window class stucture
winclass.style			= CS_DBLCLKS | CS_OWNDC | 
                          CS_HREDRAW | CS_VREDRAW;
winclass.lpfnWndProc	= WindowProc;
winclass.cbClsExtra		= 0;
winclass.cbWndExtra		= 0;
winclass.hInstance		= hinstance;
winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
winclass.lpszMenuName	= NULL;
winclass.lpszClassName	= WINDOW_CLASS_NAME;

// register the window class
if (!RegisterClass(&winclass))
	return(0);

// create the window
if (!(hwnd = CreateWindow(WINDOW_CLASS_NAME, // class
						  "Out of Time",	 // title
						  WS_OVERLAPPEDWINDOW | WS_VISIBLE,
					 	  0,0,	   // x,y
						  WINDOW_WIDTH,  // width
                          WINDOW_HEIGHT, // height
						  NULL,	   // handle to parent 
						  NULL,	   // handle to menu
						  hinstance,// instance
						  NULL)))	// creation parms
return(0);

// save the window handle in a global
main_window_handle = hwnd;

// enter main event loop
while(1)
	{
    // get current time

	if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
		// test if this is a quit
        if (msg.message == WM_QUIT)
           break;
	
		// translate any accelerator keys
		TranslateMessage(&msg);

		// send the message to the window proc
		DispatchMessage(&msg);
		} // end if

    // main game processing goes here /////////////////////
    
    // get the graphics device context 
    hdc = GetDC(hwnd);
    
    // release the device context
    ReleaseDC(hwnd,hdc);

    // wait until exatly 1 millisecond has passed from
    // the time the event loop body was entered

    Sleep(10);

    } // end while

// return to Windows like this
return(msg.wParam);

} // end WinMain

///////////////////////////////////////////////////////////

