
// PROG15_2.CPP - DirectSetup demo with callback

// INCLUDES ///////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN  
#define INITGUID

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>

#include <dsetup.h>

// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINXCLASS"  // class name

#define WINDOW_WIDTH    320   // size of window
#define WINDOW_HEIGHT   240

// number of DirectSetup return values as currently defined
// in DirectX 5.0
#define NUM_DSETUP_RVALUES             14

// MACROS /////////////////////////////////////////////////

// these read the keyboard asynchronously
#define KEY_DOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEY_UP(vk_code)   ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)

// PROTOTYPES /////////////////////////////////////////////

// game console
int Game_Init(void *parms=NULL);
int Game_Shutdown(void *parms=NULL);
int Game_Main(void *parms=NULL);

// GLOBALS ////////////////////////////////////////////////

HWND main_window_handle           = NULL; // save the window handle
HINSTANCE main_instance           = NULL; // save the instance
char buffer[80];                          // used to print text

// exact path of contents of REDIST\
// note the use of "\\" instead of "\"
// this is neccessary since "\" is an escape character
char DIRECTX_REDIST_PATH[]=".\\REDIST";

// a lookup table that holds the return values
DWORD dsetup_rvalues[] = {DSETUPERR_SUCCESS,
                          DSETUPERR_SUCCESS_RESTART,
                          DSETUPERR_BADSOURCESIZE,
                          DSETUPERR_BADSOURCETIME, 
                          DSETUPERR_BADWINDOWSVERSION,
                          DSETUPERR_CANTFINDDIR,
                          DSETUPERR_CANTFINDINF,
                          DSETUPERR_INTERNAL,
                          DSETUPERR_NOCOPY,
                          DSETUPERR_NOTPREINSTALLEDONNT,
                          DSETUPERR_OUTOFDISKSPACE,
                          DSETUPERR_SOURCEFILENOTFOUND,
                          DSETUPERR_UNKNOWNOS,
                          DSETUPERR_USERHITCANCEL };

// another lookup table so the return values can be converted
// into a string for display
char *dsetup_rstrings[] = {"DSETUPERR_SUCCESS",
                           "DSETUPERR_SUCCESS_RESTART",
                           "DSETUPERR_BADSOURCESIZE" ,
                           "DSETUPERR_BADSOURCETIME", 
                           "DSETUPERR_BADWINDOWSVERSION" ,
                           "DSETUPERR_CANTFINDDIR" ,
                           "DSETUPERR_CANTFINDINF" ,
                           "DSETUPERR_INTERNAL" ,
                           "DSETUPERR_NOCOPY" ,
                           "DSETUPERR_NOTPREINSTALLEDONNT" ,
                           "DSETUPERR_OUTOFDISKSPACE" ,
                           "DSETUPERR_SOURCEFILENOTFOUND" ,
                           "DSETUPERR_UNKNOWNOS" ,
                           "DSETUPERR_USERHITCANCEL" };

// FUNCTIONS //////////////////////////////////////////////

LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT	ps;		   // used in WM_PAINT
HDC			hdc;	   // handle to a device context

// what is the message 
switch(msg)
	{	
	case WM_CREATE: 
        {
		// do initialization stuff here
		return(0);
		} break;

    case WM_PAINT:
         {
         // start painting
         hdc = BeginPaint(hwnd,&ps);

         // end painting
         EndPaint(hwnd,&ps);
         return(0);
        } break;

	case WM_DESTROY: 
		{
		// kill the application			
		PostQuitMessage(0);
		return(0);
		} break;

	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc

// WINMAIN ////////////////////////////////////////////////

int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{
// this is the winmain function

WNDCLASS winclass;	// this will hold the class we create
HWND	 hwnd;		// generic window handle
MSG		 msg;		// generic message
HDC      hdc;       // generic dc
PAINTSTRUCT ps;     // generic paintstruct

// first fill in the window class stucture
winclass.style			= CS_DBLCLKS | CS_OWNDC | 
                          CS_HREDRAW | CS_VREDRAW;
winclass.lpfnWndProc	= WindowProc;
winclass.cbClsExtra		= 0;
winclass.cbWndExtra		= 0;
winclass.hInstance		= hinstance;
winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
winclass.lpszMenuName	= NULL; 
winclass.lpszClassName	= WINDOW_CLASS_NAME;

// register the window class
if (!RegisterClass(&winclass))
	return(0);

// create the window, note the use of WS_POPUP
if (!(hwnd = CreateWindow(WINDOW_CLASS_NAME, // class
                          "DUMB Setup Parent Window",	 // title
                          WS_OVERLAPPEDWINDOW | WS_VISIBLE,
					 	  0,0,	   // x,y
						  WINDOW_WIDTH,  // width
                          WINDOW_HEIGHT, // height
						  NULL,	   // handle to parent 
						  NULL,	   // handle to menu
						  hinstance,// instance
						  NULL)))	// creation parms
return(0);

// save the window handle and instance in a global
main_window_handle = hwnd;
main_instance      = hinstance;

// perform all game console specific initialization
Game_Init();

// enter main event loop
while(1)
	{
	if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
		// test if this is a quit
        if (msg.message == WM_QUIT)
           break;
	
		// translate any accelerator keys
		TranslateMessage(&msg);

		// send the message to the window proc
		DispatchMessage(&msg);
		} // end if
    
    // main game processing goes here
    Game_Main();

	} // end while

// shutdown game and release all resources
Game_Shutdown();

// return to Windows like this
return(msg.wParam);

} // end WinMain

// WINX GAME PROGRAMMING CONSOLE FUNCTIONS ////////////////

// this prototype is very important!
DWORD WINAPI DSetupCallback(DWORD Reason, 
                            DWORD MsgType, 
                            char *szMessage, 
                            char *szName, 
                            void *pUpgradeInfo)
{
// this is the most generic callback function you can have
// it simply returns IDOK for status calls, otherwise
// it pops up a messagebox and allows you to make the
// decision

if (MsgType==0)        
    return IDOK;

// call the messagebox function and return its value
// remember, DirectSetup is designed to respond to the
// return values of MessageBox

return(MessageBox(main_window_handle, szMessage, 
                  "DirectX Setup Demo -- Running", MsgType));

} // end DSetupCallback
 
///////////////////////////////////////////////////////////

int Game_Init(void *parms)
{
// this function is where you do all the initialization 
// for your game

char messbuffer[256]; // used to print messages

// tell user whats up
MessageBox(main_window_handle,
  "This simple application installs the\nDirectX Run-Time libraries.\n\nPress OK to Continue.\n",
  "DirectX Setup Demo -- Starting...",MB_OK);

// set callback function
DirectXSetupSetCallback(DSetupCallback);

// call directsetup
DWORD dsetup_result = 
DirectXSetup(main_window_handle,DIRECTX_REDIST_PATH,DSETUP_DIRECTX);

// test the return value and print out the DirectSetup
// error string

for (int index=0; index<NUM_DSETUP_RVALUES; index++)
    {
    if (dsetup_rvalues[index] == dsetup_result)
        {
        // build up message
        sprintf(messbuffer,"DirectSetup Has Completed.\n\nResult=%s\n\nPress OK to Continue.",dsetup_rstrings[index]);

        // print results of DirectXSetup to user
        MessageBox(main_window_handle,
                   messbuffer,
                   "DirectX Setup Demo -- Final Results",MB_OK);
        break;
        } // end if
    } // end for

// return success
return(1);

} // end Game_Init

///////////////////////////////////////////////////////////

int Game_Shutdown(void *parms)
{
// this function is where you shutdown your game and
// release all resources that you allocated

// shut everything down




// return success
return(1);
} // end Game_Shutdown

///////////////////////////////////////////////////////////

int Game_Main(void *parms)
{
// this is the workhorse of your game it will be called
// continuously in real-time this is like main() in C
// all the calls for you game go here!

// check of user is trying to exit
if (KEY_DOWN(VK_ESCAPE))
    PostMessage(main_window_handle, WM_DESTROY,0,0);



// return success
return(1);

} // end Game_Main

//////////////////////////////////////////////////////////